#! /usr/bin/env python
from utils import prompt
from colorama import init as colorama_init
from builtins import input

def main():
  a = input(prompt('Enter something: '))

if __name__ == "__main__":
  main()
